import { BaseEvent } from '#application/base/baseEvent'
import { UUID } from '#application/types'
import TileRepository from '#repositories/tileRepository'

type Payload = {
  id: UUID
  name: string
  tags: string[]
}

export default class TileUpdateEvent extends BaseEvent {
  public listen(): void {
    this.socket.on('gm:tile:update', this.handleEvent.bind(this))
  }

  private async handleEvent(data: Payload, callback: (success: boolean) => void): Promise<void> {
    if (!(await this.isCharacterGM())) return

    try {
      const tile = await TileRepository.getById(data.id)
      if (!tile) return callback(false)

      await tile.setName(data.name).setTags(data.tags).update()
      return callback(true)
    } catch (error) {
      return callback(false)
    }
  }
}
