import { BaseRepository } from '#application/base/baseRepository'
import { UUID } from '#application/types'
import { Character } from '#entities/character'

class CharacterRepository extends BaseRepository {
  async getByUserId(userId: UUID, populate?: any): Promise<Character[]> {
    try {
      const repository = this.em.getRepository(Character)
      return await repository.find({ user: userId }, { populate })
    } catch (error: any) {
      this.logger.error(`Failed to get character by user ID: ${error instanceof Error ? error.message : String(error)}`)
      return []
    }
  }

  async getByUserAndId(userId: UUID, characterId: UUID, populate?: any): Promise<Character | null> {
    try {
      const repository = this.em.getRepository(Character)
      return await repository.findOne({ user: userId, id: characterId }, { populate })
    } catch (error: any) {
      this.logger.error(`Failed to get character by user ID and character ID: ${error instanceof Error ? error.message : String(error)}`)
      return null
    }
  }

  async getById(id: UUID, populate?: string[]): Promise<Character | null> {
    try {
      const repository = this.em.getRepository(Character)
      return await repository.findOne({ id })
    } catch (error: any) {
      this.logger.error(`Failed to get character by ID: ${error instanceof Error ? error.message : String(error)}`)
      return null
    }
  }

  async getByName(name: string): Promise<Character | null> {
    try {
      const repository = this.em.getRepository(Character)
      return await repository.findOne({ name })
    } catch (error: any) {
      this.logger.error(`Failed to get character by name: ${error instanceof Error ? error.message : String(error)}`)
      return null
    }
  }
}

export default new CharacterRepository()
