import { FilterValue } from '@mikro-orm/core'

import { BaseRepository } from '#application/base/baseRepository'
import { UUID } from '#application/types'
import { unduplicateArray } from '#application/utilities'
import { Tile } from '#entities/tile'
import { Map } from '#entities/map'
import MapService from '#services/mapService'

class TileRepository extends BaseRepository {
  async getById(id: UUID) {
    try {
      const repository = this.em.getRepository(Tile)
      return await repository.findOne({ id })
    } catch (error: any) {
      return null
    }
  }

  async getByIds(ids: UUID[]) {
    try {
      const repository = this.em.getRepository(Tile)
      return await repository.find({
        id: ids
      })
    } catch (error: any) {
      return []
    }
  }

  async getAll() {
    try {
      const repository = this.em.getRepository(Tile)
      return await repository.findAll()
    } catch (error: any) {
      return []
    }
  }

  async getByMapId(mapId: UUID) {
    try {
      const repository = this.em.getRepository(Map)
      const tileRepository = this.em.getRepository(Tile)

      const map = await repository.findOne({ id: mapId })
      if (!map) return []

      const mapTileArray = unduplicateArray(MapService.flattenMapArray(JSON.parse(JSON.stringify(map.tiles))))

      return await tileRepository.find({
        id: mapTileArray
      })
    } catch (error: any) {
      return []
    }
  }
}

export default new TileRepository()
